#include "common.h"
#include "udma.h"
#include "udma_if.h"
#include "hw_mcspi.h"
static uint8_t tx_buffer[TR_BUFF_SIZE];
static uint8_t rx_buffer[TR_BUFF_SIZE];
int transfer_complete = 0;

static void interrupt_handler()
{
	uint32_t status = MAP_SPIIntStatus(GSPI_BASE,true);
	MAP_SPIIntClear(GSPI_BASE,SPI_INT_EOW);
	Message("Transfer complete!\n\r");
	SPICSDisable(GSPI_BASE);
	transfer_complete = 1;
}

void setup()
{
	// configure SPI
	MAP_SPIReset(GSPI_BASE);

	UDMAInit();

    MAP_SPIConfigSetExpClk(GSPI_BASE,MAP_PRCMPeripheralClockGet(PRCM_GSPI),
                     SPI_IF_BIT_RATE,SPI_MODE_MASTER,SPI_SUB_MODE_0,
                     (SPI_SW_CTRL_CS |
                     SPI_4PIN_MODE |
                     SPI_TURBO_OFF |
                     SPI_CS_ACTIVEHIGH |
                     SPI_WL_8));	
	MAP_SPIIntRegister(GSPI_BASE,interrupt_handler);

	MAP_SPIFIFOLevelSet( GSPI_BASE, 1, 1 );
	MAP_SPIFIFOEnable( GSPI_BASE, SPI_RX_FIFO );
    MAP_SPIFIFOEnable( GSPI_BASE, SPI_TX_FIFO ); 
	MAP_SPIDmaEnable(GSPI_BASE,SPI_RX_DMA);
	MAP_SPIDmaEnable(GSPI_BASE,SPI_TX_DMA);
	MAP_SPIIntEnable( GSPI_BASE, SPI_INT_EOW );
	MAP_SPIEnable(GSPI_BASE);

}

void spi_transfer()
{
	transfer_complete = 0;
	SetupTransfer(UDMA_CH30_GSPI_RX,UDMA_MODE_BASIC,TR_BUFF_SIZE,
                UDMA_SIZE_8,UDMA_ARB_1,
                (void *)(GSPI_BASE + MCSPI_O_RX0),UDMA_SRC_INC_NONE,
                rx_buffer,UDMA_DST_INC_8);

	SetupTransfer(UDMA_CH31_GSPI_TX,UDMA_MODE_BASIC,TR_BUFF_SIZE,
                UDMA_SIZE_8,UDMA_ARB_1,
                tx_buffer,UDMA_SRC_INC_8,
				(void *)(GSPI_BASE + MCSPI_O_TX0),UDMA_DST_INC_NONE);	
	MAP_SPIWordCountSet( GSPI_BASE, TR_BUFF_SIZE );
	SPICSEnable(GSPI_BASE);
}
int main()
{
	uint32_t rx_sum = 0;
	uint32_t tx_sum = 0;
	uint8_t tmp;
	uint32_t tx_crc = 0;
	uint32_t tx_crc_old = 0;
	uint32_t a = 364;
	uint32_t b = 53457;
	uint32_t c = 976;
	uint32_t d = 66;


	init();
	// initialize buffers with some known value
	memset(tx_buffer,0x33,sizeof(tx_buffer));
	memset(rx_buffer,0x44,sizeof(rx_buffer));

	// wait for keystroke: this gives the user time to start the slave
    Report("Start the slave and press any key to transmit data....\n\r");
    MAP_UARTCharGet(UARTA0_BASE);
	
	setup();

	tx_crc = fill_buffer(tx_buffer,TR_BUFF_SIZE,"TX");

	while(1)
	{
		tx_crc_old = tx_crc;
		change_seed(a++,b++,c++,d++);
		tx_crc = fill_buffer(tx_buffer,TR_BUFF_SIZE,"TX");
		Message("Sending...");

		spi_transfer();
		while(!transfer_complete)
		{}

		rx_sum = 0;
		for(int i = 0; i < TR_BUFF_SIZE; i++)
		{
			rx_sum += rx_buffer[i];
		}
		Report("\n\rChecksum in the RX buffer was: 0x%02x, we send 0x%02x\n\r\n\r",crc(rx_buffer),tx_crc_old);
		//Report("The sum in the Rx buffer is: %d, tx was : %d\n\r",rx_sum,tx_sum);
		Message("TX-");
		print_buffer(tx_buffer,TR_BUFF_SIZE);
		Message("RX-");
		print_buffer(rx_buffer,TR_BUFF_SIZE);		
		Report("Press any key to transmit again\n\r");
		MAP_UARTCharGet(UARTA0_BASE);
	}
	return 0;
}

